'use strict';

const toHex = d => d.reduce((hex, byte) => hex + byte.toString(16).padStart(2, '0'), '');
const fromHex = hex => {
  if (!hex.length) return new Uint8Array(0)
  return new Uint8Array(hex.match(/../g).map(b => parseInt(b, 16)))
};

const create = function base16 (alphabet) {
  return {
    encode: input => toHex(input),
    decode (input) {
      for (const char of input) {
        if (alphabet.indexOf(char) < 0) {
          throw new Error('invalid base16 character')
        }
      }
      return fromHex(input)
    }
  }
};

var base16 = { prefix: 'f', name: 'base16', ...create('0123456789abcdef') };

module.exports = base16;
