'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var baseX = _interopDefault(require('base-x'));
var buffer = require('buffer');

const TypedArray = Object.getPrototypeOf(Int8Array);
const isTypedArray = obj => obj instanceof TypedArray;

const coerce = o => {
  if (o instanceof Uint8Array && o.constructor.name === 'Uint8Array') return o
  if (o instanceof ArrayBuffer) return new Uint8Array(o)
  if (o instanceof DataView || isTypedArray(o)) {
    return new Uint8Array(o.buffer, o.byteOffset, o.byteLength)
  }
  throw new Error('Unknown type, must be binary type')
};

const wrap = obj => ({
  encode: b => obj.encode(buffer.Buffer.from(b)),
  decode: s => coerce(obj.decode(s))
});

const btc = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz';
const flickr = '123456789abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ';

var base58 = [
  { name: 'base58btc', prefix: 'z', ...wrap(baseX(btc)) },
  { name: 'base58flickr', prefix: 'Z', ...wrap(baseX(flickr)) }
];

module.exports = base58;
